
--######################################################################################--
-- Addon Data & Setting Defaults
--######################################################################################--

-- root namespace so collisions aren't a problem
-- I put everything under the root local "Enforcer" variable
local Enforcer = {}

-- addon data
Enforcer.version = 2.0;								-- completely arbitrary
Enforcer.lastUpdateTime = Inspect.System.Time()
Enforcer.context = UI.CreateContext("Enforcer")  	-- set us up the ux
Enforcer.masterBuffList = {}						-- superset of all buffs being monitored
Enforcer.activeBuffList = {}						-- subset of master list based on abilities available
Enforcer.updateInterval = .2						-- refresh rate in seconds

-- refresh triggers
Enforcer.buffsChanged = true						-- used when buff events occur
Enforcer.targetChanged = true						-- used when buff events occur

-- common settings
Enforcer.settings = {};		

-- should set this to a default here TODO later
Enforcer.settings.version = 1.0					-- settings contract version, this should change when setting data becomes incompatible
Enforcer.settings.debug = false

-- buffs frame settings
Enforcer.settings["buffs"] = {}
Enforcer.settings["buffs"].x = 600					-- assumes 1600x1200 res, probably a bad assumption, but w/e
Enforcer.settings["buffs"].y = 400

-- target frame settings
Enforcer.settings["target"] = {}
Enforcer.settings["target"].x = 900
Enforcer.settings["target"].y = 400



--######################################################################################--
-- Buff data tables (this may change between game updates)
--######################################################################################--

-- list of incoming healing debuffs
Enforcer.incomingHealDebuffs = { 
	--["Vex"] 				= 50, 				-- this is just for testing
	--["Sanction"] 			= 30,				-- this is just for testing
	["Essence Link"]		= 30,				-- unconfirmed		mage
	["Lingering Wounds"]	= 50,				-- unconfirmed		warrior
	["Void Shroud"]			= 50,				-- unconfirmed		mage
	["Spiteful Spirit"]		= 50,				-- confirmed		cleric
	["Retribution"]			= 50,				-- confirmed		cleric
	["Withered Veins"]		= 5,				-- unconfirmed		mage, STACKS
	["Trauma"]				= 50,				-- unconfirmed		warrior
	["Fell Blades"]			= 50,				-- unconfirmed		rogue
	["Maul"]				= 30,				-- unconfirmed		rogue pet
	["Blight"]				= 50,				-- unconfirmed		mage
}

-- list of outgoing healing debuffs
Enforcer.outgoingHealDebuffs = {
	--["Sanction"] 			= 30,				-- this is just for testing
	["Anathema"]			= 10,				-- unconfirmed		rogue, STACKS
}

-- list of persistent healing effect buffs
Enforcer.healingEffects = {
	["Soothing Stream"]				=	1,
	["Healing Current"]				=	1,
	["Overflow"]					=	1,
	["Healing Spray"]				=	1,
	["Healing Flood"]				=	1,
	["Healing Showers"]				=	1,
	["Hailstone Tideshard"]			=	1,
	["Grasping Faesource"]			=	1,
	["Orbs of the Tide"]			=	1,
	["Orbs of the Stream"]			=	1,
	["Vigilance"]					=	1,
	["Ward of the Ancestors"]		=	1,
	["Latent Blaze"]				=	1,
	["Spiritual Conflagration"]		=	1,
	["Ancestral Flame"]				=	1,
	["Healing Blessing"]			=	1,
	["Caregiver&apos;s Blessing"]	=	1,		-- might get mistaken for the self-buff from a purifier, need a way to sort this
	["Rite of the Ancestors"]		=	1,
	["Shield of the Ancestors"]		=	1,
	["Righteous Mandate"]			=	1,		-- not really a healing effect but indicates the target is likely to be healed
	["Synthesis"]					=	1,		-- not really a healing effect but indicates the target is likely to be healed
	["Surging Flames"]				=	1,		-- not really a healing effect but indicates the target is likely to be healed
	["Lasting Invocation"]			=	1,
	["Frozen Deep One Tear"]		=	1,
	["Protect the Flock"]			=	1,		-- not a heal but indicates the target is being healed by AOE & is harder to kill
	["Crystal Vine"]				=	1,
}

-- list of defensive effect buffs
-- TODO add it

--######################################################################################--
-- UI Stuff
--######################################################################################--

-- generic function to create the icon style I like
local function createIcon (r, g, b, a)

	-- create the base icon
	buffIcon = UI.CreateFrame("Texture", "Icon", Enforcer.context)
	buffIcon:SetHeight(36)
	buffIcon:SetWidth(36)
	buffIcon:SetBackgroundColor(r, g, b, a)		
	buffIcon:SetLayer(1)
	
	-- create icon texture, center it over the icon and make it smaller
	buffIcon.icon = UI.CreateFrame("Texture", "IconTexture", buffIcon)	
	buffIcon.icon:SetPoint("CENTER", buffIcon, "CENTER")
	buffIcon.icon:SetHeight(32)
	buffIcon.icon:SetWidth(32)
	buffIcon.icon:SetLayer(2)
	
	-- create icon text
	buffIcon.text = UI.CreateFrame("Text", "IconText", buffIcon)
	buffIcon.text:SetFontSize(20)
	buffIcon.text:SetFontColor(1,1,0,1)
	buffIcon.text:SetPoint("CENTER", buffIcon, "CENTER")
	buffIcon.text:SetLayer(3)

	-- hide the icon by default
	buffIcon:SetVisible(false)
	
	-- function to set text, resize, and re-anchor
	function buffIcon:Label(text)
		if text then self.text:SetText(tostring(text)) else self.text:SetText("") end
		self.text:ResizeToText()
		self.text:SetPoint("CENTER", self, "CENTER")
	end
	
	-- function to set image to an ability name
	function buffIcon:SetAbility(id)
		local ability = Inspect.Ability.Detail(id)
		if not ability then return end
		self.icon:SetTexture("Rift", ability.icon)
		self.id = id
	end
	
	-- function to set texture
	function buffIcon:Texture(texture)
		if not texture then return end
		self.icon:SetTexture("Rift", texture)
	end
	
	-- return the icon
	return buffIcon;
end


--######################################################################################--
-- Buff Watch Feature
--######################################################################################--

-- following Zorba's design on saving frames for re-use since frames cannot be destroyed
Enforcer.buffsActiveFrame = {}
Enforcer.buffsZombieFrame = {}
local buffsLastAttach = nil

-- this creates a buff icon for the buffs frame
-- it pulls from the existing objects in the zombie frame first
-- then creates new frames when required
-- anchor point is stored in settings
function Enforcer.buffsAddBuff(abilityID)
	-- pull from zombie resevoir of frames if possible
	local buffIcon = table.remove(Enforcer.buffsZombieFrame)
	if not buffIcon then
		buffIcon = createIcon(.7, 0, 0, 1)
	end
	buffIcon:SetAbility(abilityID)
	buffIcon:SetVisible(true)
	table.insert(Enforcer.buffsActiveFrame, buffIcon)
	
	-- attach the buffIcon to an anchor point
	if not buffsLastAttach then
		-- if no attach its the first anchor, so put it where the settings say to
		buffIcon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Enforcer.settings["buffs"].x, Enforcer.settings["buffs"].y);
	else
		-- otherwise anchor it next to the previous one
		buffIcon:SetPoint("TOPLEFT", buffsLastAttach, "TOPRIGHT", 2, 0);
	end
	buffsLastAttach = buffIcon;
end

-- refreshes the buffs indicator
-- gets missing buffs and then adds them
function Enforcer.buffsRefresh()
	-- Get the list of buffs from the player. 
	-- this seems inefficient, grumble grumble
	local buffList = Inspect.Buff.List("player")	
	if not buffList then return end 
	local buffDetails = Inspect.Buff.Detail("player", buffList)
	
	-- copy the active buff list
	local missingBuffs = {}
	for name, id in pairs(Enforcer.activeBuffList) do
		missingBuffs[name] = id
	end

	-- go through current buffs, subtracting any matches from the copy list
	for id, buff in pairs(buffDetails) do
		if (missingBuffs[buff.name] ~= nil) then
			missingBuffs[buff.name] = nil
		end
	end
	
	-- entries that remain are missing buffs
	-- just a debug string to see what we're returning here
	if (Enforcer.settings.debug) then 
		print("DEBUG: Missing Buffs:")
		for name, id in pairs(missingBuffs) do
			if (Enforcer.settings.debug) then print(string.format("DEBUG: \t%s", name)) end
		end
	end

	-- clear out the active buff frame, save away the buff frames 
	-- (there is no way to delete a frame, can likely get rid of this once they are handled properly)
	for _, v in pairs(Enforcer.buffsActiveFrame) do
		v:SetVisible(false)
		-- we must reset any points we use or it will cause problems later
		-- this is a little bitch and has really odd behavior if it doesn't undo the attach points
		v:SetPoint("TOPLEFT", UIParent, "TOPCENTER")	
		table.insert(Enforcer.buffsZombieFrame, v)
	end
	Enforcer.buffsActiveFrame = {}
	buffsLastAttach = nil
	
	-- redraw the buffs frame with the missing buffs
	for name, value in pairs(missingBuffs) do
		Enforcer.buffsAddBuff(value)
	end
end

-- trigger buff change
function Enforcer.buffsTrigger(unit, buffs)
	if not unit or Inspect.Unit.Lookup(unit) == "player" then
		Enforcer.buffsChanged = true
	end
end
-- add to buff events
table.insert(Event.Buff.Add, {Enforcer.buffsTrigger, "Enforcer", "Buff Watch"})
table.insert(Event.Buff.Change, {Enforcer.buffsTrigger, "Enforcer", "Buff Watch"})
table.insert(Event.Buff.Remove, {Enforcer.buffsTrigger, "Enforcer", "Buff Watch"})



--######################################################################################--
-- Status Scanning
--######################################################################################--

-- Function to scan a specifier and update the indicated icons
-- Used by target status feature and player status feature
function Enforcer.statusRefresh(specifier, inhealIcon, outhealIcon, healeffectIcon)
	local buffList = Inspect.Buff.List(specifier)

	-- clear out frame
	inhealIcon:SetVisible(false)
	outhealIcon:SetVisible(false)
	healeffectIcon:SetVisible(false)
	
	-- if no target bug out
	if not buffList then return	end
	
	-- tracking
	local healEffectCount = 0
	local maxInhealDebuff = 0
	local maxOuthealDebuff = 0
	
	-- scan buffs
	local buffDetails = Inspect.Buff.Detail(specifier, buffList)
	for id, buff in pairs(buffDetails) do
	
		-- incoming heal debuff
		if (Enforcer.incomingHealDebuffs[buff.name] ~= nil) then
			if Enforcer.settings.debug then print(string.format("Target is incoming Heal debuffed: %s  - by %s", Enforcer.incomingHealDebuffs[buff.name], buff.caster)) end
			local inVal = Enforcer.incomingHealDebuffs[buff.name]
			if buff.stack then
				inVal = inVal * buff.stack
			end
			
			-- compare to max debuff thus far seen
			if inVal > maxInhealDebuff then
				inhealIcon:Label(inVal)
				maxInhealDebuff = inVal
			end
			inhealIcon:SetVisible(true)
		end
		
		-- outgoing heal debuff
		if (Enforcer.outgoingHealDebuffs[buff.name] ~= nil) then
			if Enforcer.settings.debug then print(string.format("Target is outgoing Heal debuffed: %s  - by %s", Enforcer.outgoingHealDebuffs[buff.name], buff.caster)) end
			local outVal = Enforcer.outgoingHealDebuffs[buff.name]
			if buff.stack then
				outVal = outVal * buff.stack
			end
			if outVal > maxOuthealDebuff then
				outhealIcon:Label(outVal)
				maxOuthealDebuff = outVal
			end
			outhealIcon:SetVisible(true)
		end
		
		-- big damage reduction
		
		
		-- evasion / immunities
		
		
		-- Heal Effects (note some heal effect names match debuff names, so must not include debuffs)
		if not buff.debuff and Enforcer.healingEffects[buff.name] ~= nil then
		if Enforcer.settings.debug then print(string.format("Target has healing effect: %s  - by %s", Enforcer.outgoingHealDebuffs[buff.name], buff.caster)) end
			local healVal = Enforcer.healingEffects[buff.name]
			if buff.stack then
				healVal = healVal + buff.stack - 1		-- buff stack of nil == 1, get off-by-one if we don't subtract
			end
			-- add to existing heal effects
			healEffectCount = healEffectCount + healVal
			healeffectIcon:Label(healEffectCount)
			healeffectIcon:SetVisible(true)
		end
	end
end

--######################################################################################--
-- Target Status Feature
--######################################################################################--
-- This feature is designed to capture the relevant PvP information on the player's
-- target, including incoming heal debuff effects, outgoing heal debuff effects, number of
-- healing effects on the player, and number number of players targeting them. The purpose
-- of this feature is to identify weak targets and focus targets and know when to lay on
-- the burst / cc. Eventually this feature will also include special buff effects but I'm
-- trying to cut down on the UI clutter that this might cause. For now it's Healing Debuffs,
-- healing effects, and number of friendlies targetting. I plan on adding buff watching for
-- both defensive and offensive cooldowns but need to figure out an easy way to convey this
-- in the UX.

-- TEXTURE FORMAT "Data/\\UI\\ability_icons\\righteousness1.dds"

-- Create the Icons

-- Enforcer incoming heal debuff icon
Enforcer.targetInhealDebuffIcon = createIcon(.4,0,0,1)		
Enforcer.targetInhealDebuffIcon:Texture("Data/\\UI\\ability_icons\\drainlife3.dds")
Enforcer.targetInhealDebuffIcon.text:SetFontColor(1,1,0,1)
Enforcer.targetInhealDebuffIcon.icon:SetAlpha(.6)

-- Enforcer outgoing heal debuff icon
Enforcer.targetOuthealDebuffIcon = createIcon(.3,0,.5,1)		
Enforcer.targetOuthealDebuffIcon:Texture("Data/\\UI\\ability_icons\\channelhealth3.dds")
Enforcer.targetOuthealDebuffIcon.text:SetFontColor(1,1,0,1)
Enforcer.targetOuthealDebuffIcon.icon:SetAlpha(.6)

-- Enforcer healing effect icon
Enforcer.targetHealEffectIcon = createIcon(0,0,.4,1)
Enforcer.targetHealEffectIcon:Texture("Data/\\UI\\ability_icons\\warden-deluge.dds")
Enforcer.targetHealEffectIcon.text:SetFontColor(1,1,0,1)
Enforcer.targetHealEffectIcon.icon:SetAlpha(.4)

-- This is a function so they can be re-anchored after changing settings
function Enforcer.anchorIcons()
	-- heal debuff is the root anchor
	Enforcer.targetInhealDebuffIcon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Enforcer.settings["target"].x, Enforcer.settings["target"].y)
	-- place outgoing heal above incoming heal
	Enforcer.targetOuthealDebuffIcon:SetPoint("BOTTOMLEFT", Enforcer.targetInhealDebuffIcon, "TOPLEFT", 0, -2)
	-- place healing indicator to the right of the heal debuff
	Enforcer.targetHealEffectIcon:SetPoint("TOPLEFT", Enforcer.targetInhealDebuffIcon, "TOPRIGHT", 2, 0)
end

-- trigger the refresh
function Enforcer.targetTrigger(unit, buffs)
	if not unit or Inspect.Unit.Lookup(unit) == "player.target" then
		Enforcer.targetChanged = true
		if Enforcer.settings.debug then print("Target changed!") end
	end
end
-- add to buff events
table.insert(Event.Buff.Add, {Enforcer.targetTrigger, "Enforcer", "Target Refresh"})
table.insert(Event.Buff.Change,{Enforcer.targetTrigger, "Enforcer", "Target Refresh"})
table.insert(Event.Buff.Remove,{Enforcer.targetTrigger, "Enforcer", "Target Refresh"})


--######################################################################################--
-- Data refresh methods
--######################################################################################--

-- Update events for when the ability list changes, need to change the current view
-- This needs to be called everytime the data is changed or the ability list changes.
-- update the active buff list from the master based on current abilities
function Enforcer.updateActiveBuffList ()
	-- clear active buff list
	Enforcer.activeBuffList = {}
	
	-- get ability list
	local abilities = Inspect.Ability.List()
	if (not abilities) then return end
	local abilityDetails = Inspect.Ability.Detail(abilities)
	
	-- loop through all active abilities
	for _, ability in pairs(abilityDetails) do
		local name = ability.name
		-- if ability is in master list, add it to active list
		if (Enforcer.masterBuffList[name] ~= nil) then
			Enforcer.activeBuffList[name] = Enforcer.masterBuffList[name]
		end
	end
	
	-- reset triggers
	Enforcer.buffsTrigger()
	Enforcer.targetTrigger()
	
	if (Enforcer.settings.debug) then print("DEBUG: Enforcer active buff list updated.") end
end
-- Add event for updating the active list when abilities change
table.insert(Event.Ability.Availability.Add, {Enforcer.updateActiveBuffList, "Enforcer", "Update Active Buff List"})
table.insert(Event.Ability.Availability.Remove, {Enforcer.updateActiveBuffList, "Enforcer", "Update Active Buff List"})


--######################################################################################--
-- Main Pulse Function																	--
--######################################################################################--

-- function that does the buff checking.
-- is throttled by both time & event triggers
function Enforcer.refresh()
	-- throttle our calls to the updateInterval so we dont' eat up shit tons of cpu.
	local currentTime = Inspect.System.Time()
	if (currentTime - Enforcer.lastUpdateTime < Enforcer.updateInterval) then return end
	Enforcer.lastUpdateTime = currentTime
	
	-- Buff Watch
	if (Enforcer.buffsChanged) then
		-- do the refresh 
		Enforcer.buffsRefresh()
		Enforcer.buffsChanged = false
	end
	
	-- Target Watch
	if (Enforcer.targetChanged) then
		-- do the refresh
		Enforcer.statusRefresh("player.target", Enforcer.targetInhealDebuffIcon, Enforcer.targetOuthealDebuffIcon, Enforcer.targetHealEffectIcon)
		Enforcer.targetChanged = false
	end
	
	-- TODO: Add Player Watch so the player knows his own status
	
end
-- register event for updating
table.insert(Event.System.Update.Begin, {Enforcer.refresh, "Enforcer", "Periodic Update"})



--######################################################################################--
-- Data Save & Load																		--
--######################################################################################--

-- saves state to saved variables
function Enforcer.saveState(addon)
	-- saved variables returns addon
	if (addon ~= "Enforcer") then return end
	
	if (Enforcer.settings.debug) then print("DEBUG: Saving Enforcer State") end
	
	-- save the master buff list
	Enforcer_masterBuffList = Enforcer.masterBuffList
	-- save settings
	Enforcer_settings = Enforcer.settings
	
	if (Enforcer.settings.debug) then print("DEBUG: Enforcer State Saved") end
end
-- save state event
table.insert(Event.Addon.SavedVariables.Save.Begin, {Enforcer.saveState, "Enforcer", "Save state"})


-- loads state from saved variables & does other app initialization
function Enforcer.loadState(addon)
	-- saved variables returns addon
	if (addon ~= "Enforcer") then return end
	
	if (Enforcer.settings.debug) then print("DEBUG: Loading Enforcer State") end
	
	-- if the contract changes, settings go to default. I'm not even going to try to migrate them.
	-- will try not to change the contract in the future, but hey, we don't know these things when we first make addons...
	-- version check is exact, lower contract versions and higher contract versions cannot be guaranteed to match
	-- if they do not match then we'll get all sorts of load errors after updating the addon.
	-- need to check for .version because I released a version without it
	if (Enforcer_settings and Enforcer_settings.version and Enforcer_settings.version == 1.0) then
		-- load settings
		Enforcer.settings = Enforcer_settings;
		-- load master buff list
		if (Enforcer_masterBuffList) then
			Enforcer.masterBuffList = Enforcer_masterBuffList;
		end
	end
	
	-- update the active list 
	Enforcer.updateActiveBuffList()
	
	-- anchor the icons
	Enforcer.anchorIcons ()

	if (Enforcer.settings.debug) then print("DEBUG: Enforcer State Loaded") end
end
-- load state event
table.insert(Event.Addon.SavedVariables.Load.End, {Enforcer.loadState, "Enforcer", "Load state"})



--######################################################################################--
-- Command line options and utilities													--
--######################################################################################--

-- strips off the first word of the command line, leaves the rest for future parsing
-- probably useful in other mods, this is...consider for library project
local function nextToken (str)
	local first, rest
 	_, _, first, rest = string.find(str, "^(%w+)%s?(.*)$")
	return first, rest
end

-- Print usage
function Enforcer.opt_help ()
	print("Enforcer Usage:")
	print("\tlist\t\t\t\tLists all monitored abilities.")
	print("\tadd <ability>\t\tAdd an ability to the monitored list.")
	print("\tdel <ability>\t\tDelete an ability from the monitored list.")
	print("\tmove <frame> <x> <y>\tMove frame anchor to (x,y) from topleft. Frame values: buffs, target")
	print("\tdebug\t\t\tToggle debug mode.")
	print("\thelp\t\t\t\tThis usage message.")
end

-- List abilities in the monitor list
-- TODO: add updated output for abilities that includes the buff warning timer, if specified
function Enforcer.opt_list()
	print("Master list of monitored abilities:")
	for name, id in pairs(Enforcer.masterBuffList) do
		print(string.format("\tAbility: %s", name))
	end
	print("ACTIVE monitored abilities:")
	for name, id in pairs(Enforcer.activeBuffList) do
		print(string.format("\tAbility: %s", name))
	end
end

-- Add an ability to the monitor list
-- TODO: add optional argument to add command to specify warning timer
-- TODO: change format of the buff data to be an object.
function Enforcer.opt_add(args)
	-- get ability list
	local abilities = Inspect.Ability.List()
	if (not abilities) then return end
	local abilityDetails = Inspect.Ability.Detail(abilities)
	
	-- loop through all active abilities
	for id, ability in pairs(abilityDetails) do
		local name = ability.name
		if (name == args) then
			Enforcer.masterBuffList[name] = id
			print(string.format("Ability %s added.", name))
			-- update the active list
			Enforcer.updateActiveBuffList()
			return;
		end
	end
	print(string.format("Ability not found: %s\nMake sure the ability is available in the current role.", args))
	
end

-- update warning timer on buff
-- TODO: add this function

-- Delete an ability from the monitor list
function Enforcer.opt_del(args)
	if (Enforcer.masterBuffList[args] ~= nil) then
		Enforcer.masterBuffList[args] = nil;
		print(string.format("Ability %s deleted!", args))
		-- update the active list
		Enforcer.updateActiveBuffList()
		return
	end
	print(string.format("Ability not found: %s\nCheck /enforcer list for the correct listing of monitored abilities.", args))
end

-- move around the enforcer frames
-- takes the form: <frame> <x> <y>
-- where <frame> is one of the frames (buffs, target, etc)
function Enforcer.opt_move(args)
	local frame, pos = nextToken(args)
	
	-- check frame for valid option
	if (frame ~= "buffs" and frame ~= "target") then
		print(string.format("Invalid option: %s. Use 'buffs' or 'target'", frame or ""))
		return;
	end
	
	-- validate input data and update appropriate frame
	local x, y = pos:match("^(%d+)%s+(%d+)$")
	x, y = tonumber(x), tonumber(y)
	if (x and y) then
		Enforcer.settings[frame].x = x
		Enforcer.settings[frame].y = y
		print(string.format("Enforcer %s frame moved to %d, %d", frame, Enforcer.settings[frame].x, Enforcer.settings[frame].y))
	else
		print("Move requires two positive integers as input.")
	end
	-- update the new anchors
	Enforcer.anchorIcons ()
end

-- Toggle debug
function Enforcer.opt_debug()
	if (Enforcer.settings.debug) then
		Enforcer.settings.debug = false
		print("Enforcer Debug Disabled.")
	else
		Enforcer.settings.debug = true
		print("Enforcer Debug Enabled.")
	end
end

-- root command parser
-- be nice to have a library for this shit, put on to-do list
function Enforcer.cmd_Enforcer (args)
	local opt, args = nextToken(args)
	if (opt == "list") then
		Enforcer.opt_list()
	elseif (opt == "add") then
		Enforcer.opt_add(args)
	elseif (opt == "del") then
		Enforcer.opt_del(args)
	elseif (opt == "debug") then
		Enforcer.opt_debug()
	elseif (opt == "move") then
		Enforcer.opt_move(args)
	else
		Enforcer.opt_help()
	end
end
-- slash command registration, use full name and the short "enf" for the lazy bastards like myself
table.insert(Command.Slash.Register("enforcer"),{Enforcer.cmd_Enforcer, "Enforcer", "Slash command"})
table.insert(Command.Slash.Register("enf"),{Enforcer.cmd_Enforcer, "Enforcer", "Slash command"})


--######################################################################################--
-- Use LibUnitChange
--######################################################################################--

-- have to do it this way because there isn't a dependency model yet...
function Enforcer.LoadLibs (addon)
	if addon == "LibUnitChange" then
		-- detect target change
		Library.UnitChange("Enforcer", {"player.target"}, function() Enforcer.targetChanged = true end)
		
		if Enforcer.settings.debug then print("DEBUG: LibUnitChange Loaded!") end
	end
end
table.insert(Event.Addon.Load.End, {Enforcer.LoadLibs, "Enforcer", "Enforcer lib"})



